############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.4
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
############################################################
import logging
import PyQt5
import time
from queue import Queue
from astrometry import transform


class MountStatusRunnerOnce(PyQt5.QtCore.QObject):
    logger = logging.getLogger(__name__)

    signalDestruct = PyQt5.QtCore.pyqtSignal()
    CYCLE = 250
    CYCLE_STATUS_ONCE = 500
    CONNECTION_TIMEOUT = 2000

    def __init__(self, app, thread, data, signalConnected, mountStatus):
        super().__init__()

        self.app = app
        self.thread = thread
        self.data = data
        self.signalConnected = signalConnected
        self.mountStatus = mountStatus
        self.mutexIsRunning = PyQt5.QtCore.QMutex()
        self.dataTimer = None
        self.cycleTimer = None
        self.isRunning = False
        self.connectCounter = 0
        self.socket = None
        self.sendLock = False
        self.messageString = ''
        self.sendCommandQueue = Queue()
        self.transform = transform.Transform(self.app)

    def run(self):
        self.logger.info('mount once started')
        self.mutexIsRunning.lock()
        if not self.isRunning:
            self.isRunning = True
        self.mutexIsRunning.unlock()
        self.socket = PyQt5.QtNetwork.QTcpSocket()
        self.socket.setSocketOption(PyQt5.QtNetwork.QAbstractSocket.LowDelayOption, 1)
        self.socket.setSocketOption(PyQt5.QtNetwork.QAbstractSocket.KeepAliveOption, 1)
        self.socket.hostFound.connect(self.handleHostFound)
        self.socket.connected.connect(self.handleConnected)
        self.socket.stateChanged.connect(self.handleStateChanged)
        self.socket.disconnected.connect(self.handleDisconnect)
        self.socket.readyRead.connect(self.handleReadyRead)
        self.socket.error.connect(self.handleError)
        # timers
        self.dataTimer = PyQt5.QtCore.QTimer(self)
        self.dataTimer.setSingleShot(False)
        self.dataTimer.timeout.connect(self.getStatusOnce)
        self.dataTimer.start(self.CYCLE_STATUS_ONCE)
        self.signalDestruct.connect(self.destruct, type=PyQt5.QtCore.Qt.BlockingQueuedConnection)
        self.cycleTimer = PyQt5.QtCore.QTimer(self)
        self.cycleTimer.setSingleShot(False)
        self.cycleTimer.timeout.connect(self.doCommand)
        self.cycleTimer.start(self.CYCLE)

    def stop(self):
        self.mutexIsRunning.lock()
        if self.isRunning:
            self.isRunning = False
            self.signalDestruct.emit()
            self.signalConnected.emit({'Once': False})
            self.thread.quit()
            self.thread.wait()
        self.mutexIsRunning.unlock()
        self.logger.info('mount once stopped')

    @PyQt5.QtCore.pyqtSlot()
    def destruct(self):
        if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
            self.socket.disconnectFromHost()
        self.cycleTimer.stop()
        self.dataTimer.stop()
        self.signalDestruct.disconnect(self.destruct)
        self.socket.hostFound.disconnect(self.handleHostFound)
        self.socket.connected.disconnect(self.handleConnected)
        self.socket.stateChanged.disconnect(self.handleStateChanged)
        self.socket.disconnected.disconnect(self.handleDisconnect)
        self.socket.error.disconnect(self.handleError)
        self.socket.readyRead.disconnect(self.handleReadyRead)
        self.socket.abort()

    def doCommand(self):
        self.doReconnect()
        if not self.sendCommandQueue.empty() and (self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState):
            command = self.sendCommandQueue.get()
            if not self.sendLock:
                self.sendCommand(command)

    def doReconnect(self):
        if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.UnconnectedState:
            if self.connectCounter == 0:
                self.app.sharedMountDataLock.lockForRead()
                self.socket.connectToHost(self.data['MountIP'], self.data['MountPort'])
                self.app.sharedMountDataLock.unlock()
                self.sendCommandQueue.queue.clear()
            else:
                # connection build up is ongoing
                pass
            if self.connectCounter * self.CYCLE > self.CONNECTION_TIMEOUT:
                self.socket.abort()
                self.connectCounter = 0
            else:
                self.connectCounter += 1
        else:
            if self.socket.state() != PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
                if self.connectCounter * self.CYCLE > self.CONNECTION_TIMEOUT:
                    self.socket.abort()
                    self.connectCounter = 0
                else:
                    self.connectCounter += 1
            else:
                # connected
                pass

    @PyQt5.QtCore.pyqtSlot()
    def handleHostFound(self):
        self.app.sharedMountDataLock.lockForRead()
        self.logger.debug('Mount RunnerOnce found at {}:{}'.format(self.data['MountIP'], self.data['MountPort']))
        self.app.sharedMountDataLock.unlock()

    @PyQt5.QtCore.pyqtSlot()
    def handleConnected(self):
        self.app.sharedMountDataLock.lockForRead()
        self.logger.info('Mount RunnerOnce connected at {0}:{1}'.format(self.data['MountIP'], self.data['MountPort']))
        self.app.sharedMountDataLock.unlock()
        self.signalConnected.emit({'Once': True})

    @PyQt5.QtCore.pyqtSlot(PyQt5.QtNetwork.QAbstractSocket.SocketError)
    def handleError(self, socketError):
        if self.socket.error() > 0:
            self.logger.warning('Mount RunnerOnce connection fault: {0}'.format(socketError))

    @PyQt5.QtCore.pyqtSlot()
    def handleStateChanged(self):
        self.logger.debug('Mount RunnerOnce connection has state: {0}'.format(self.socket.state()))

    @PyQt5.QtCore.pyqtSlot()
    def handleDisconnect(self):
        self.logger.info('Mount RunnerOnce connection is disconnected from host')
        self.signalConnected.emit({'Once': False})

    def sendCommand(self, command):
        if self.isRunning:
            if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
                self.socket.write(bytes(command + '\r', encoding='ascii'))
                self.socket.flush()
            else:
                self.logger.warning('Socket RunnerOnce not connected')

    def getStatusOnce(self):
        if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
            command = ':U2#:Gev#:Gg#:Gt#:GVD#:GVN#:GVP#:GVT#:GVZ#:newalig#:endalig#'
            # command = ':U2#:Gev#:Gg#:Gt#:GVD#:GVN#:GVP#:GVT#:GVZ#'
            self.sendCommandQueue.put(command)
            self.dataTimer.stop()

    @PyQt5.QtCore.pyqtSlot()
    def handleReadyRead(self):
        messageToProcess = ''
        # Get message from socket.
        while self.socket.bytesAvailable() and self.isRunning:
            self.messageString += self.socket.read(1024).decode()
            if len(self.messageString.strip('#').split('#')) < 10:
                return
            else:
                messageToProcess = self.messageString
                self.messageString = ''
        # now transfer the model data
        try:
            if len(messageToProcess) == 0:
                return
            self.app.sharedMountDataLock.lockForWrite()
            valueList = messageToProcess.strip('#').split('#')
            # +0580.9#-011:42:17.3#+48:02:01.6#Oct 25 2017#2.15.8#10micron GM1000HPS#16:58:31#Q-TYPE2012#
            # all parameters are delivered
            self.logger.info('Once raw: {0}'.format(messageToProcess))
            self.logger.info('Once processed: {0}'.format(valueList))
            if len(valueList) >= 8:
                if len(valueList[0]) > 0:
                    self.data['SiteHeight'] = valueList[0]
                if len(valueList[1]) > 0:
                    lon1 = valueList[1]
                    # due to compatibility to LX200 protocol east is negative
                    if lon1[0] == '-':
                        self.data['SiteLongitude'] = lon1.replace('-', '+')
                    else:
                        self.data['SiteLongitude'] = lon1.replace('+', '-')
                if len(valueList[2]) > 0:
                    self.data['SiteLatitude'] = valueList[2]
                if len(valueList[3]) > 0:
                    self.data['FirmwareDate'] = valueList[3]
                if len(valueList[4]) > 0:
                    self.data['FirmwareNumber'] = valueList[4]
                    fw = self.data['FirmwareNumber'].split('.')
                    if len(fw) == 3:
                        self.data['FW'] = int(float(fw[0]) * 10000 + float(fw[1]) * 100 + float(fw[2]))
                    else:
                        self.data['FW'] = 0
                if len(valueList[5]) > 0:
                    self.data['FirmwareProductName'] = valueList[5]
                if len(valueList[6]) > 0:
                    self.data['FirmwareTime'] = valueList[6]
                if len(valueList[7]) > 0:
                    self.data['HardwareVersion'] = valueList[7]
                self.logger.info('FW: {0} Number: {1}'.format(self.data['FirmwareNumber'], self.data['FW']))
                self.logger.info('Site Lon:    {0}'.format(self.data['SiteLongitude']))
                self.logger.info('Site Lat:    {0}'.format(self.data['SiteLatitude']))
                self.logger.info('Site Height: {0}'.format(self.data['SiteHeight']))
                self.app.signalMountSiteData.emit(self.data['SiteLatitude'], self.data['SiteLongitude'], self.data['SiteHeight'])
            else:
                self.logger.warning('Parsing Status Once combined command valueList is not OK: length:{0} content:{1}'.format(len(valueList), valueList))
        except Exception as e:
            self.logger.error('Problem parsing response, error: {0}, message:{1}'.format(e, messageToProcess))
        finally:
            self.app.sharedMountDataLock.unlock()
        self.sendLock = False



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.4
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import logging
import time
import indi.indi_xml as indiXML


class INDIEnvironment:
    logger = logging.getLogger(__name__)
    START_ENVIRONMENT_TIMEOUT = 3

    def __init__(self, main, app, data):
        # make main sources available
        self.main = main
        self.app = app
        self.data = data
        self.application = dict()
        self.application['Available'] = False
        self.application['Name'] = 'INDI Environment'
        self.application['Status'] = ''

    def start(self):
        self.connect()

    def stop(self):
        pass

    def connect(self):
        timeStart = time.time()
        while True:
            if time.time() - timeStart > self.START_ENVIRONMENT_TIMEOUT:
                self.app.messageQueue.put('Timeout connect environment device\n')
                break
            if self.app.workerINDI.environmentDevice:
                if 'CONNECTION' in self.app.workerINDI.data['Device'][self.app.workerINDI.environmentDevice]:
                    break
            time.sleep(0.1)
        if self.app.workerINDI.environmentDevice != '':
            if self.app.workerINDI.data['Device'][self.app.workerINDI.environmentDevice]['CONNECTION']['CONNECT'] == 'Off':
                self.app.INDICommandQueue.put(indiXML.newSwitchVector([indiXML.oneSwitch('On', indi_attr={'name': 'CONNECT'})], indi_attr={'name': 'CONNECTION', 'device': self.app.workerINDI.environmentDevice}))

    def getStatus(self):
        if self.app.workerINDI.environmentDevice != '':
            self.application['Available'] = True
        else:
            self.application['Available'] = False
        self.app.sharedEnvironmentDataLock.lockForWrite()
        if self.app.workerINDI.environmentDevice in self.app.workerINDI.data['Device']:
            self.application['Status'] = 'OK'
            self.data['Connected'] = self.app.workerINDI.data['Device'][self.app.workerINDI.environmentDevice]['CONNECTION']['CONNECT'] == 'On'
        else:
            self.data['Connected'] = False
            self.application['Status'] = 'ERROR'
        self.app.sharedEnvironmentDataLock.unlock()

    def getData(self):
        # check if client has device found
        if self.app.workerINDI.environmentDevice != '':
            # and device is connected
            if self.app.workerINDI.data['Device'][self.app.workerINDI.environmentDevice]['CONNECTION']['CONNECT'] == 'On':
                # than get the data
                self.app.sharedEnvironmentDataLock.lockForWrite()
                if 'WEATHER_DEWPOINT' in self.app.workerINDI.data['Device'][self.app.workerINDI.environmentDevice]['WEATHER_PARAMETERS']:
                    self.data['DewPoint'] = float(self.app.workerINDI.data['Device'][self.app.workerINDI.environmentDevice]['WEATHER_PARAMETERS']['WEATHER_DEWPOINT'])
                else:
                    self.data['DewPoint'] = 0
                if 'WEATHER_TEMPERATURE' in self.app.workerINDI.data['Device'][self.app.workerINDI.environmentDevice]['WEATHER_PARAMETERS']:
                    self.data['Temperature'] = float(self.app.workerINDI.data['Device'][self.app.workerINDI.environmentDevice]['WEATHER_PARAMETERS']['WEATHER_TEMPERATURE'])
                else:
                    self.data['Temperature'] = 0
                if 'WEATHER_HUMIDITY' in self.app.workerINDI.data['Device'][self.app.workerINDI.environmentDevice]['WEATHER_PARAMETERS']:
                    self.data['Humidity'] = float(self.app.workerINDI.data['Device'][self.app.workerINDI.environmentDevice]['WEATHER_PARAMETERS']['WEATHER_HUMIDITY'])
                else:
                    self.data['Humidity'] = 0
                if 'WEATHER_BAROMETER' in self.app.workerINDI.data['Device'][self.app.workerINDI.environmentDevice]['WEATHER_PARAMETERS']:
                    self.data['Pressure'] = float(self.app.workerINDI.data['Device'][self.app.workerINDI.environmentDevice]['WEATHER_PARAMETERS']['WEATHER_BAROMETER'])
                else:
                    self.data['Pressure'] = 0
                self.app.sharedEnvironmentDataLock.unlock()

        # check if client has device SQM found
        if self.app.workerINDI.auxDevice != '':
            if self.app.workerINDI.data['Device'][self.app.workerINDI.auxDevice]['CONNECTION']['CONNECT'] == 'On':
                # than get the data
                self.app.sharedEnvironmentDataLock.lockForWrite()
                if 'SKY_BRIGHTNESS' in self.app.workerINDI.data['Device'][self.app.workerINDI.auxDevice]['SKY_QUALITY']:
                    self.data['SQR'] = float(self.app.workerINDI.data['Device'][self.app.workerINDI.auxDevice]['SKY_QUALITY']['SKY_BRIGHTNESS'])
                else:
                    self.data['SQR'] = 0
                self.app.sharedEnvironmentDataLock.unlock()



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.4
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
############################################################
import logging
import PyQt5
import time
from queue import Queue
from astrometry import transform


class MountStatusRunnerFast(PyQt5.QtCore.QObject):
    logger = logging.getLogger(__name__)

    CONNECTION_TIMEOUT = 2000
    CYCLE_STATUS_FAST = 750
    CYCLE = 250
    signalDestruct = PyQt5.QtCore.pyqtSignal()

    def __init__(self, app, thread, data, signalConnected, mountStatus):
        super().__init__()

        self.app = app
        self.data = data
        self.thread = thread
        self.signalConnected = signalConnected
        self.mountStatus = mountStatus
        self.mutexIsRunning = PyQt5.QtCore.QMutex()
        self.dataTimer = None
        self.cycleTimer = None
        self.isRunning = False
        self.connectCounter = 0
        self.socket = None
        self.sendLock = False
        self.messageString = ''
        self.sendCommandQueue = Queue()
        self.transform = transform.Transform(self.app)
        self.audioDone = False

    def run(self):
        self.logger.info('mount fast started')
        self.mutexIsRunning.lock()
        if not self.isRunning:
            self.isRunning = True
        self.mutexIsRunning.unlock()
        self.socket = PyQt5.QtNetwork.QTcpSocket()
        self.socket.setSocketOption(PyQt5.QtNetwork.QAbstractSocket.LowDelayOption, 1)
        self.socket.setSocketOption(PyQt5.QtNetwork.QAbstractSocket.KeepAliveOption, 1)
        self.socket.hostFound.connect(self.handleHostFound)
        self.socket.connected.connect(self.handleConnected)
        self.socket.stateChanged.connect(self.handleStateChanged)
        self.socket.disconnected.connect(self.handleDisconnect)
        self.socket.readyRead.connect(self.handleReadyRead)
        self.socket.error.connect(self.handleError)
        # timers
        self.dataTimer = PyQt5.QtCore.QTimer(self)
        self.dataTimer.setSingleShot(False)
        self.dataTimer.timeout.connect(self.getStatusFast)
        self.dataTimer.start(self.CYCLE_STATUS_FAST)
        self.signalDestruct.connect(self.destruct, type=PyQt5.QtCore.Qt.BlockingQueuedConnection)
        self.cycleTimer = PyQt5.QtCore.QTimer(self)
        self.cycleTimer.setSingleShot(False)
        self.cycleTimer.timeout.connect(self.doCommand)
        self.cycleTimer.start(self.CYCLE)

    def stop(self):
        self.mutexIsRunning.lock()
        if self.isRunning:
            self.isRunning = False
            self.signalDestruct.emit()
            self.signalConnected.emit({'Fast': False})
            self.thread.quit()
            self.thread.wait()
        self.mutexIsRunning.unlock()
        self.logger.info('mount fast stopped')

    @PyQt5.QtCore.pyqtSlot()
    def destruct(self):
        if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
            self.socket.disconnectFromHost()
        self.cycleTimer.stop()
        self.dataTimer.stop()
        self.socket.hostFound.disconnect(self.handleHostFound)
        self.socket.connected.disconnect(self.handleConnected)
        self.socket.stateChanged.disconnect(self.handleStateChanged)
        self.socket.disconnected.disconnect(self.handleDisconnect)
        self.socket.error.disconnect(self.handleError)
        self.socket.readyRead.disconnect(self.handleReadyRead)
        self.signalDestruct.disconnect(self.destruct)
        self.socket.abort()

    def doCommand(self):
        self.doReconnect()
        if not self.sendCommandQueue.empty() and (self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState):
            command = self.sendCommandQueue.get()
            if not self.sendLock:
                self.sendCommand(command)

    def doReconnect(self):
        # to get order in connections, we wait for first connecting the once type
        if self.mountStatus['Once'] and self.data['FW'] > 0:
            if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.UnconnectedState:
                if self.connectCounter == 0:
                    self.app.sharedMountDataLock.lockForRead()
                    self.socket.connectToHost(self.data['MountIP'], self.data['MountPort'])
                    self.app.sharedMountDataLock.unlock()
                    self.sendCommandQueue.queue.clear()
                else:
                    # connection build up is ongoing
                    pass
                if self.connectCounter * self.CYCLE > self.CONNECTION_TIMEOUT:
                    self.socket.abort()
                    self.connectCounter = 0
                else:
                    self.connectCounter += 1
            else:
                if self.socket.state() != PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
                    if self.connectCounter * self.CYCLE > self.CONNECTION_TIMEOUT:
                        self.socket.abort()
                        self.connectCounter = 0
                    else:
                        self.connectCounter += 1
                else:
                    # connected
                    pass

    @PyQt5.QtCore.pyqtSlot()
    def handleHostFound(self):
        self.app.sharedMountDataLock.lockForRead()
        self.logger.debug('Mount RunnerFast found at {}:{}'.format(self.data['MountIP'], self.data['MountPort']))
        self.app.sharedMountDataLock.unlock()

    @PyQt5.QtCore.pyqtSlot()
    def handleConnected(self):
        self.signalConnected.emit({'Fast': True})
        self.app.sharedMountDataLock.lockForRead()
        self.logger.info('Mount RunnerFast connected at {0}:{1}'.format(self.data['MountIP'], self.data['MountPort']))
        self.app.sharedMountDataLock.unlock()

    @PyQt5.QtCore.pyqtSlot(PyQt5.QtNetwork.QAbstractSocket.SocketError)
    def handleError(self, socketError):
        self.logger.warning('Mount RunnerFast connection fault: {0}'.format(socketError))

    @PyQt5.QtCore.pyqtSlot()
    def handleStateChanged(self):
        self.logger.debug('Mount RunnerFast connection has state: {0}'.format(self.socket.state()))

    @PyQt5.QtCore.pyqtSlot()
    def handleDisconnect(self):
        self.logger.info('Mount RunnerFast connection is disconnected from host')
        self.signalConnected.emit({'Fast': False})

    def sendCommand(self, command):
        if self.isRunning:
            if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
                self.sendLock = True
                self.socket.write(bytes(command + '\r', encoding='ascii'))
                self.socket.flush()
            else:
                self.sendLock = False
                self.logger.warning('Socket RunnerFast not connected')

    @PyQt5.QtCore.pyqtSlot()
    def getStatusFast(self):
        if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
            self.sendCommandQueue.put(':U2#:GS#:Ginfo#:')

    @PyQt5.QtCore.pyqtSlot()
    def handleReadyRead(self):
        # Get message from socket.
        while self.socket.bytesAvailable() and self.isRunning:
            self.messageString += self.socket.read(1024).decode()
        if len(self.messageString) < 71:
            return
        else:
            messageToProcess = self.messageString[:71]
            self.messageString = self.messageString[71:]
        # Try and parse the message. In Fast we ask for GS and Ginfo so we expect 2
        try:
            if len(messageToProcess) == 0:
                return
            self.app.sharedMountDataLock.lockForWrite()
            valueList = messageToProcess.strip('#').split('#')
            # first the GS command
            if len(valueList) == 2:
                if len(valueList[0]) > 0:
                    self.data['LocalSiderealTime'] = valueList[0].strip('#')
                # second the Ginfo
                if len(valueList[1]) > 0:
                    value = ''
                    try:
                        value = valueList[1].rstrip('#').strip().split(',')
                        if len(value) == 8:
                            self.data['RaJNow'] = float(value[0])
                            self.data['DecJNow'] = float(value[1])
                            self.data['Pierside'] = str(value[2])
                            self.data['Az'] = float(value[3])
                            self.data['Alt'] = float(value[4])
                            self.data['JulianDate'] = str(value[5])
                            self.data['Status'] = int(value[6])
                            # if stop , emit warning
                            if value[6] in ['1', '98', '99']:
                                # only emit one time !
                                if not self.audioDone:
                                    self.app.audioCommandQueue.put('MountAlert')
                                    self.app.messageQueue.put('#BRMount STOPPED - WARNING !!!\n')
                                    # stop any modeling
                                    self.app.workerModelingDispatcher.signalCancel.emit()
                                self.audioDone = True
                            else:
                                self.audioDone = False
                            # calculate if slewing stopped
                            if 'Slewing' in self.data:
                                if self.data['Slewing'] and value[7] != '1':
                                    self.app.workerMountDispatcher.signalSlewFinished.emit()
                                    self.app.audioCommandQueue.put('MountSlew')
                            self.data['Slewing'] = (value[7] == '1')
                            self.data['RaJ2000'], self.data['DecJ2000'] = self.transform.transformERFA(self.data['RaJNow'], self.data['DecJNow'], 2)
                            self.data['TelescopeRA'] = '{0}'.format(self.transform.decimalToDegree(self.data['RaJ2000'], False, False))
                            self.data['TelescopeDEC'] = '{0}'.format(self.transform.decimalToDegree(self.data['DecJ2000'], True, False))
                            self.data['TelescopeAltitude'] = '{0:03.2f}'.format(self.data['Alt'])
                            self.data['TelescopeAzimuth'] = '{0:03.2f}'.format(self.data['Az'])
                            self.data['MountStatus'] = '{0}'.format(self.data['Status'])
                            if self.data['Pierside'] == str('W'):
                                self.data['TelescopePierSide'] = 'WEST'
                            else:
                                self.data['TelescopePierSide'] = 'EAST'
                            self.app.workerMountDispatcher.signalMountAzAltPointer.emit(self.data['Az'], self.data['Alt'])
                            self.app.signalJulianDate.emit(float(self.data['JulianDate']))
                        else:
                            self.logger.warning('Ginfo command delivered wrong number of arguments: {0}'.format(value))
                    except Exception as e:
                        self.logger.error('Receive error Ginfo command: {0} reply:{1}'.format(e, value))
                    finally:
                        pass
            else:
                self.logger.warning('Parsing GS-Ginfo combined command valueList is not OK: length:{0} content:{1}'.format(len(valueList), valueList))
        except Exception as e:
            self.logger.error('Problem parsing response, error: {0}, message:{1}'.format(e, messageToProcess))
        finally:
            self.app.sharedMountDataLock.unlock()
        self.sendLock = False



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.4
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import logging
import platform
if platform.system() == 'Windows':
    from win32com.client.dynamic import Dispatch
    import pythoncom


class AscomDome:
    logger = logging.getLogger(__name__)

    def __init__(self, main, app, data):
        # make main sources available
        self.main = main
        self.app = app
        self.data = data
        self.ascom = None
        self.chooser = None
        self.driverName = ''
        self.application = dict()
        self.application['Available'] = True
        self.application['Name'] = 'Ascom Dome'
        self.application['Status'] = ''

    def start(self):
        pythoncom.CoInitialize()
        if self.driverName != '' and self.ascom is None:
            try:
                self.ascom = Dispatch(self.driverName)
                self.ascom.connected = True
                self.logger.info('Driver chosen:{0}'.format(self.driverName))
                self.application['Status'] = 'OK'
                self.app.sharedDomeDataLock.lockForWrite()
                self.data['Connected'] = True
                self.app.sharedDomeDataLock.unlock()
                self.logger.info('ASCOM dome started')
            except Exception as e:
                self.application['Status'] = 'ERROR'
                self.logger.error('Could not dispatch driver: {0} and connect it, error: {1}'.format(self.driverName, e))
            finally:
                pass
        elif self.driverName == '':
            # no connection made
            self.app.sharedDomeDataLock.lockForWrite()
            self.data['Connected'] = False
            self.app.sharedDomeDataLock.unlock()
            self.application['Status'] = 'ERROR'
            self.logger.info('ASCOM Dome could not be started')

    def stop(self):
        try:
            if self.ascom:
                self.ascom.connected = False
        except Exception as e:
            self.logger.error('Could not stop driver: {0} and close it, error: {1}'.format(self.driverName, e))
        finally:
            self.app.sharedDomeDataLock.lockForWrite()
            self.data['Connected'] = False
            self.app.sharedDomeDataLock.unlock()
            self.ascom = None
            self.logger.info('ASCOM Dome stopped')
            pythoncom.CoUninitialize()

    def slewToAzimuth(self, azimuth):
        if not self.ascom:
            return
        if not self.ascom.connected:
            return
        try:
            self.ascom.SlewToAzimuth(float(azimuth))
        except Exception as e:
            self.logger.error('Problem slewing azimuth, error: {0}'.format(e))
        finally:
            pass

    def getStatus(self):
        try:
            if self.ascom:
                if self.ascom.connected:
                    self.app.sharedDomeDataLock.lockForWrite()
                    self.data['Connected'] = True
                    self.app.sharedDomeDataLock.unlock()
                else:
                    self.app.sharedDomeDataLock.lockForWrite()
                    self.data['Connected'] = False
                    self.app.sharedDomeDataLock.unlock()
                self.application['Status'] = 'OK'
            else:
                self.app.sharedDomeDataLock.lockForWrite()
                self.data['Connected'] = False
                self.app.sharedDomeDataLock.unlock()
        except Exception as e:
            self.logger.error('Could not dispatch driver: {0} and connect it, error: {1}'.format(self.driverName, e))
            self.app.sharedDomeDataLock.lockForWrite()
            self.data['Connected'] = False
            self.app.sharedDomeDataLock.unlock()
            self.application['Status'] = 'ERROR'
        finally:
            pass

    def getData(self):
        if not self.ascom:
            return
        if not self.ascom.connected:
            return
        self.app.sharedDomeDataLock.lockForWrite()
        try:
            if self.data['Slewing'] and not self.ascom.Slewing:
                self.main.signalSlewFinished.emit()
                self.app.audioCommandQueue.put('DomeSlew')
            self.data['Slewing'] = self.ascom.Slewing
        except Exception as e:
            self.logger.error('Problem getting data, error: {0}'.format(e))
        finally:
            pass
        try:
            self.data['Azimuth'] = self.ascom.Azimuth
        except Exception as e:
            self.logger.error('Problem getting data, error: {0}'.format(e))
        finally:
            pass
        #try:
        #    self.data['Altitude'] = self.ascom.Altitude
        #except Exception as e:
        #    self.logger.error('Problem getting data, error: {0}'.format(e))
        #finally:
        #    pass
        self.app.sharedDomeDataLock.unlock()

    def setupDriver(self):
        try:
            self.chooser = Dispatch('ASCOM.Utilities.Chooser')
            self.chooser.DeviceType = 'Dome'
            self.driverName = self.chooser.Choose(self.driverName)
            self.app.messageQueue.put('Driver chosen:{0}\n'.format(self.driverName))
            self.logger.info('Driver chosen:{0}'.format(self.driverName))
        except Exception as e:
            self.app.messageQueue.put('#BRDriver error in Setup Driver\n')
            self.logger.error('General error:{0}'.format(e))
        finally:
            pass



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.5
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import logging
import PyQt5
from baseclasses import widget
from astrometry import transform
import astropy
import copy
import numpy
import matplotlib
import bisect
from gui import hemisphere_window_ui


class HemisphereWindow(widget.MwWidget):
    logger = logging.getLogger(__name__)

    def __init__(self, app):
        super(HemisphereWindow, self).__init__()
        self.app = app
        self.transform = transform.Transform(self.app)
        self.mutexDrawCanvas = PyQt5.QtCore.QMutex()
        self.mutexDrawCanvasMoving = PyQt5.QtCore.QMutex()

        self.pointerAzAlt1 = None
        self.pointerAzAlt2 = None
        self.pointerAzAlt3 = None
        self.pointerDome1 = None
        self.pointerDome2 = None
        self.pointerTrack = None
        self.pointsPlotBig = None
        self.pointsPlotSmall = None
        self.pointsPlotCross = None
        self.starsAlignment = None
        self.starsAnnotate = list()
        self.maskPlotFill = None
        self.maskPlotMarker = None
        self.deltaGuide = None
        self.deltaSlew = None
        self.celestial = None
        self.annotate = list()
        self.offx = 1
        self.offy = 1
        self.ui = hemisphere_window_ui.Ui_HemisphereDialog()
        self.ui.setupUi(self)
        self.initUI()
        # allow sizing of the window
        self.setFixedSize(PyQt5.QtCore.QSize(16777215, 16777215))
        # set the minimum size
        self.setMinimumSize(791, 400)

        # setup the plot styles
        self.hemisphereMatplotlib = widget.IntegrateMatplotlib(self.ui.hemisphere)
        # making background looking transparent
        self.hemisphereMatplotlib.fig.patch.set_facecolor('none')
        background = self.hemisphereMatplotlib.fig.canvas.parentWidget()
        background.setStyleSheet('background-color: transparent;')
        self.hemisphereMatplotlib.axes = self.hemisphereMatplotlib.fig.add_subplot(111)
        # using tight layout because of the axis titles and labels
        self.hemisphereMatplotlib.fig.subplots_adjust(left=0.075, right=0.95, bottom=0.1, top=0.975)
        #self.hemisphereMatplotlib.fig.set_tight_layout((0.075, 0.075, 0.925, 0.925))

        # for the fast moving parts
        self.hemisphereMatplotlibMoving = widget.IntegrateMatplotlib(self.ui.hemisphereMoving)
        # making background looking transparent
        self.hemisphereMatplotlibMoving.fig.patch.set_facecolor('none')
        background = self.hemisphereMatplotlibMoving.fig.canvas.parentWidget()
        background.setStyleSheet('background-color: transparent;')
        self.hemisphereMatplotlibMoving.axes = self.hemisphereMatplotlibMoving.fig.add_subplot(111)

        # for the stars in background
        self.hemisphereMatplotlibStar = widget.IntegrateMatplotlib(self.ui.hemisphereStar)
        self.ui.hemisphereStar.setVisible(False)
        # making background looking transparent
        self.hemisphereMatplotlibStar.fig.patch.set_facecolor('none')
        background = self.hemisphereMatplotlibStar.fig.canvas.parentWidget()
        background.setStyleSheet('background-color: transparent;')
        self.hemisphereMatplotlibStar.axes = self.hemisphereMatplotlibStar.fig.add_subplot(111)

        # signal connections
        self.app.workerModelingDispatcher.signalModelPointsRedraw.connect(self.updateModelPoints)

        self.app.workerMountDispatcher.signalMountAzAltPointer.connect(self.setAzAltPointer)
        self.app.workerMountDispatcher.signalMountLimits.connect(self.updateMeridianLimits)
        self.ui.checkShowMeridian.clicked.connect(self.updateMeridianLimits)
        self.app.workerMountDispatcher.signalAlignmentStars.connect(self.updateAlignmentStars)
        self.ui.checkShowAlignmentStars.clicked.connect(self.setShowAlignmentStars)
        self.app.workerMountDispatcher.signalAlignmentStars.connect(self.updateCelestial)
        self.ui.checkShowCelestial.clicked.connect(self.updateCelestial)
        self.app.workerDome.signalDomePointer.connect(self.setDomePointer)
        self.ui.checkEditNone.clicked.connect(self.setOperationModus)
        self.ui.checkEditModelPoints.clicked.connect(self.setOperationModus)
        self.ui.checkEditHorizonMask.clicked.connect(self.setOperationModus)
        self.ui.checkPolarAlignment.clicked.connect(self.setOperationModus)

        self.ui.btn_deletePoints.clicked.connect(lambda: self.app.workerModelingDispatcher.commandDispatcher('DeletePoints'))
        self.app.workerModelingDispatcher.modelingRunner.workerSlewpoint.signalPointImaged.connect(self.plotImagedPoint)

        # from start on invisible
        self.showStatus = False
        self.setVisible(False)

    def resizeEvent(self, QResizeEvent):
        # allow message window to be resized in height
        self.ui.hemisphere.setGeometry(10, 130, self.width() - 20, self.height() - 140)
        self.ui.hemisphereStar.setGeometry(10, 130, self.width() - 20, self.height() - 140)
        self.ui.hemisphereMoving.setGeometry(10, 130, self.width() - 20, self.height() - 140)
        # getting position of axis
        axesPos = self.hemisphereMatplotlib.axes.get_position()
        # and using it fo the other plot widgets to be identically same size and position
        self.hemisphereMatplotlibStar.axes.set_position(axesPos)
        self.hemisphereMatplotlibMoving.axes.set_position(axesPos)
        # size the header window as well
        self.ui.hemisphereBackground.setGeometry(0, 0, self.width(), 126)

    def initConfig(self):
        try:
            if 'HemisphereWindowPositionX' in self.app.config:
                x = self.app.config['HemisphereWindowPositionX']
                y = self.app.config['HemisphereWindowPositionY']
                if x > self.screenSizeX:
                    x = 0
                if y > self.screenSizeY:
                    y = 0
                self.move(x, y)
            if 'HemisphereWindowShowStatus' in self.app.config:
                self.showStatus = self.app.config['HemisphereWindowShowStatus']
            if 'CheckEditNone' in self.app.config:
                self.ui.checkEditNone.setChecked(self.app.config['CheckEditNone'])
            if 'CheckEditModelPoints' in self.app.config:
                self.ui.checkEditModelPoints.setChecked(self.app.config['CheckEditModelPoints'])
            if 'CheckEditHorizonMask' in self.app.config:
                self.ui.checkEditHorizonMask.setChecked(self.app.config['CheckEditHorizonMask'])
            if 'CheckShowAlignmentStars' in self.app.config:
                self.ui.checkShowAlignmentStars.setChecked(self.app.config['CheckShowAlignmentStars'])
                self.ui.checkPolarAlignment.setDisabled(not self.app.config['CheckShowAlignmentStars'])
            if 'CheckShowCelestial' in self.app.config:
                self.ui.checkShowCelestial.setChecked(self.app.config['CheckShowCelestial'])
            if 'CheckShowMeridian' in self.app.config:
                self.ui.checkShowMeridian.setChecked(self.app.config['CheckShowMeridian'])
            if 'CheckPolarAlignment' in self.app.config:
                self.ui.checkPolarAlignment.setChecked(self.app.config['CheckPolarAlignment'])
            if 'HemisphereWindowHeight' in self.app.config and 'HemisphereWindowWidth' in self.app.config:
                self.resize(self.app.config['HemisphereWindowWidth'], self.app.config['HemisphereWindowHeight'])

        except Exception as e:
            self.logger.error('item in config.cfg not be initialize, error:{0}'.format(e))
        finally:
            pass

    def storeConfig(self):
        self.app.config['HemisphereWindowPositionX'] = self.pos().x()
        self.app.config['HemisphereWindowPositionY'] = self.pos().y()
        self.app.config['HemisphereWindowShowStatus'] = self.showStatus
        self.app.config['CheckEditNone'] = self.ui.checkEditNone.isChecked()
        self.app.config['CheckEditModelPoints'] = self.ui.checkEditModelPoints.isChecked()
        self.app.config['CheckEditHorizonMask'] = self.ui.checkEditHorizonMask.isChecked()
        self.app.config['CheckShowAlignmentStars'] = self.ui.checkShowAlignmentStars.isChecked()
        self.app.config['CheckShowCelestial'] = self.ui.checkShowCelestial.isChecked()
        self.app.config['CheckShowMeridian'] = self.ui.checkShowMeridian.isChecked()
        self.app.config['CheckPolarAlignment'] = self.ui.checkPolarAlignment.isChecked()
        self.app.config['HemisphereWindowHeight'] = self.height()
        self.app.config['HemisphereWindowWidth'] = self.width()

    def toggleWindow(self):
        self.showStatus = not self.showStatus
        if self.showStatus:
            self.showWindow()
        else:
            self.close()

    def showWindow(self):
        self.showStatus = True
        self.setVisible(True)
        self.drawHemisphere()
        self.show()
        self.app.signalChangeStylesheet.emit(self.app.ui.btn_openHemisphereWindow, 'running', 'true')

    def closeEvent(self, closeEvent):
        super().closeEvent(closeEvent)
        self.app.signalChangeStylesheet.emit(self.app.ui.btn_openHemisphereWindow, 'running', 'false')

    def selectHorizonPointsMode(self):
        msg = self.app.workerModelingDispatcher.modelingRunner.modelPoints.loadHorizonPoints(self.app.ui.le_horizonPointsFileName.text(),
                                                                                             self.app.ui.checkUseFileHorizonLine.isChecked(),
                                                                                             self.app.ui.checkUseMinimumHorizonLine.isChecked(),
                                                                                             self.app.ui.altitudeMinimumHorizon.value())
        if msg:
            self.app.messageQueue.put(msg + '\n')
        self.drawHemisphere()

    def drawCanvas(self):
        if not self.mutexDrawCanvas.tryLock():
            self.logger.warning('Performance issue in drawing')
            return
        self.hemisphereMatplotlib.fig.canvas.draw()
        if self.ui.checkShowAlignmentStars.isChecked():
            self.hemisphereMatplotlibStar.fig.canvas.draw()
        self.mutexDrawCanvas.unlock()
        PyQt5.QtWidgets.QApplication.processEvents()

    def drawCanvasMoving(self):
        if not self.mutexDrawCanvasMoving.tryLock():
            self.logger.warning('Performance issue in drawing')
            return
        self.hemisphereMatplotlibMoving.fig.canvas.draw()
        self.mutexDrawCanvasMoving.unlock()
        PyQt5.QtWidgets.QApplication.processEvents()

    def updateModelPoints(self):
        if self.showStatus:
            self.drawHemisphere()
            self.updateCelestial()
            self.updateAlignmentStars()

    def updateMeridianLimits(self):
        if self.showStatus:
            self.app.sharedMountDataLock.lockForRead()
            if 'MeridianLimitSlew' in self.app.workerMountDispatcher.data:
                slew = copy.copy(self.app.workerMountDispatcher.data['MeridianLimitSlew'])
                guide = copy.copy(self.app.workerMountDispatcher.data['MeridianLimitGuide'])
            else:
                slew = 0
                guide = 0
            self.app.sharedMountDataLock.unlock()
            self.deltaGuide.set_visible(self.ui.checkShowMeridian.isChecked())
            self.deltaSlew.set_visible(self.ui.checkShowMeridian.isChecked())
            self.deltaGuide.set_xy((180 - guide, 0))
            self.deltaSlew.set_xy((180 - slew, 0))
            self.deltaGuide.set_width(2 * guide)
            self.deltaSlew.set_width(2 * slew)
            self.drawCanvas()

    def updateCelestial(self):
        if self.showStatus:
            if self.celestial is not None:
                self.celestial.set_visible(self.ui.checkShowCelestial.isChecked())
                celestial = self.app.workerModelingDispatcher.modelingRunner.modelPoints.celestialEquator
                self.celestial.set_data([i[0] for i in celestial], [i[1] for i in celestial])
                self.drawCanvas()

    def updateAlignmentStars(self):
        if self.showStatus:
            self.ui.hemisphereStar.setVisible(self.ui.checkShowAlignmentStars.isChecked())
            starsTopo = copy.copy(self.app.workerMountDispatcher.data['starsTopo'])
            starsNames = copy.copy(self.app.workerMountDispatcher.data['starsNames'])
            if len(starsNames) > 0:
                self.starsAlignment.set_data([i[0] for i in starsTopo], [i[1] for i in starsTopo])
                for i in range(0, len(starsNames)):
                    self.starsAnnotate[i].set_position((starsTopo[i][0] + self.offx, starsTopo[i][1] + self.offy))
                self.hemisphereMatplotlibStar.fig.canvas.draw()

    def setAzAltPointer(self, az, alt):
        if self.showStatus:
            self.pointerAzAlt1.set_data((az, alt))
            self.pointerAzAlt2.set_data((az, alt))
            self.pointerAzAlt3.set_data((az, alt))
            self.pointerAzAlt1.set_visible(True)
            self.pointerAzAlt2.set_visible(True)
            self.pointerAzAlt3.set_visible(True)
            self.drawCanvasMoving()

    def setDomePointer(self, az, stat):
        if self.showStatus:
            self.pointerDome1.set_visible(stat)
            self.pointerDome2.set_visible(stat)
            self.pointerDome1.set_xy((az - 15, 1))
            self.pointerDome2.set_xy((az - 15, 1))
            self.drawCanvasMoving()

    def setShowAlignmentStars(self):
        self.ui.hemisphereStar.setVisible(self.ui.checkShowAlignmentStars.isChecked())
        if self.ui.checkShowAlignmentStars.isChecked():
            self.ui.checkPolarAlignment.setDisabled(False)
        else:
            self.ui.checkPolarAlignment.setDisabled(True)
            if self.ui.checkPolarAlignment.isChecked():
                self.ui.checkPolarAlignment.setChecked(False)
                self.ui.checkEditNone.setChecked(True)
        self.setOperationModus()

    def plotImagedPoint(self, az, alt):
        self.pointsPlotCross.set_data(numpy.append(az, self.pointsPlotCross.get_xdata()), numpy.append(alt, self.pointsPlotCross.get_ydata()))
        self.drawCanvas()

    def setOperationModus(self):
        # reset the settings
        self.maskPlotMarker.set_marker('None')
        self.maskPlotMarker.set_color('#006000')
        self.pointsPlotBig.set_color('#00A000')
        self.starsAlignment.set_color('#C0C000')
        self.starsAlignment.set_markersize(6)
        for i in range(0, len(self.starsAnnotate)):
            self.starsAnnotate[i].set_color('#808080')
        self.ui.hemisphere.stackUnder(self.ui.hemisphereMoving)
        if self.ui.checkEditNone.isChecked():
            pass
        elif self.ui.checkEditModelPoints.isChecked():
            self.pointsPlotBig.set_color('#FF00FF')
            self.ui.hemisphereMoving.stackUnder(self.ui.hemisphere)
        elif self.ui.checkEditHorizonMask.isChecked():
            self.maskPlotMarker.set_marker('o')
            self.maskPlotMarker.set_color('#FF00FF')
            self.ui.hemisphereMoving.stackUnder(self.ui.hemisphere)
        elif self.ui.checkPolarAlignment.isChecked():
            self.starsAlignment.set_color('#FFFF00')
            self.starsAlignment.set_markersize(12)
            for i in range(0, len(self.starsAnnotate)):
                self.starsAnnotate[i].set_color('#F0F0F0')
            self.ui.hemisphere.stackUnder(self.ui.hemisphereMoving)
        else:
            pass
        self.drawCanvas()

    def onMouse(self, event):
        if event.inaxes is None:
            return
        ind = None
        indlow = None
        self.app.sharedMountDataLock.lockForRead()
        stars = self.app.workerMountDispatcher.data['starsTopo']
        self.app.sharedMountDataLock.unlock()
        points = self.app.workerModelingDispatcher.modelingRunner.modelPoints.modelPoints
        horizon = self.app.workerModelingDispatcher.modelingRunner.modelPoints.horizonPoints
        if self.ui.checkEditNone.isChecked():
            # double click makes slew to target
            if event.button == 1 and event.dblclick:
                azimuth = int(event.xdata)
                altitude = int(event.ydata)
                question = 'Do you want to slew the mount to:\n\nAzimuth:\t{0}°\nAltitude:\t{1}°'.format(azimuth, altitude)
                value = self.dialogMessage(self, 'Hemisphere direct slew', question)
                if value == PyQt5.QtWidgets.QMessageBox.Ok:
                    # get tracking status of mount:
                    self.app.sharedMountDataLock.lockForRead()
                    isTracking = (self.app.workerMountDispatcher.data['Status'] == '0')
                    self.app.sharedMountDataLock.unlock()
                    # sending the commands for slewing
                    self.app.mountCommandQueue.put(':PO#')
                    self.app.mountCommandQueue.put(':Sz{0:03d}*00#'.format(azimuth))
                    self.app.mountCommandQueue.put(':Sa+{0:02d}*00#'.format(altitude))
                    self.app.mountCommandQueue.put(':MA#')
                    # and start tracking again, if mount was in tracking mode, because :MA# command stops tracking
                    if isTracking:
                        self.app.mountCommandQueue.put(':AP#')

        if self.ui.checkPolarAlignment.isChecked():
            if event.button == 1 and event.dblclick:
                print('got event')
                ind = self.get_ind_under_point(event, 2, stars)
                if ind:
                    self.app.sharedMountDataLock.lockForRead()
                    name = self.app.workerMountDispatcher.data['starsNames'][ind]
                    # RA in degrees ICRS
                    RaJ2000 = self.transform.degStringToDecimal(self.app.workerMountDispatcher.data['starsICRS'][ind][0], ' ')
                    DecJ2000 = self.transform.degStringToDecimal(self.app.workerMountDispatcher.data['starsICRS'][ind][1], ' ')
                    # correct for proper motion
                    jd_2000 = 2451544.5
                    jd_delta = float(self.app.workerMountDispatcher.data['JulianDate']) - jd_2000
                    jd_year_delta = jd_delta / 365.25
                    RaJ2000 += jd_year_delta * self.app.workerMountDispatcher.data['starsICRS'][ind][2] / 3600000
                    DecJ2000 += jd_year_delta * self.app.workerMountDispatcher.data['starsICRS'][ind][3] / 3600000
                    question = 'Do you want to slew to\npolar align star:\n\n{0}'.format(name)
                    value = self.dialogMessage(self, 'Polar Align Routine', question)
                    self.app.sharedMountDataLock.unlock()
                    if value == PyQt5.QtWidgets.QMessageBox.Ok:
                        # transform to JNOW, RAJ2000 comes in degrees, need to be hours
                        RaJNow, DecJNow = self.transform.transformERFA(RaJ2000, DecJ2000, 3)
                        RA = self.transform.decimalToDegreeMountSr(RaJNow)
                        DEC = self.transform.decimalToDegreeMountSd(DecJNow)
                        self.app.mountCommandQueue.put(':PO#')
                        self.app.mountCommandQueue.put(RA)
                        self.app.mountCommandQueue.put(DEC)
                        self.app.mountCommandQueue.put(':MS#')
                    # todo: open automatically the image window and start continuously taking pictures

        # first do the model points
        if self.ui.checkEditModelPoints.isChecked():
            ind = self.get_ind_under_point(event, 2, points)
        if self.ui.checkEditHorizonMask.isChecked():
            ind = self.get_ind_under_point(event, 2, horizon)
            indlow = self.get_two_ind_under_point_in_x(event, horizon)
        if event.button == 3 and ind is not None and self.ui.checkEditModelPoints.isChecked():
            if len(points) > 0:
                del(points[ind])
                self.annotate[ind].remove()
                del(self.annotate[ind])
            self.pointsPlotBig.set_data([i[0] for i in points], [i[1] for i in points])
            self.pointsPlotSmall.set_data([i[0] for i in points], [i[1] for i in points])
        if event.button == 1 and ind is None and self.ui.checkEditModelPoints.isChecked():
            points.append((event.xdata, event.ydata))
            if self.app.ui.checkSortPoints.isChecked():
                self.app.workerModelingDispatcher.modelingRunner.modelPoints.sortPoints()
            self.annotate.append(self.hemisphereMatplotlib.axes.annotate('', xy=(event.xdata + self.offx, event.ydata + self.offy), color='#E0E0E0'))
            self.pointsPlotBig.set_data([i[0] for i in points], [i[1] for i in points])
            self.pointsPlotSmall.set_data([i[0] for i in points], [i[1] for i in points])
        if self.ui.checkEditModelPoints.isChecked():
            for i in range(0, len(points)):
                self.annotate[i].set_text('{0:2d}'.format(i + 1))
            self.app.messageQueue.put('ToModel>{0:02d}'.format(len(points)))

        # now do the horizon mask
        if event.button == 3 and ind is not None and self.ui.checkEditHorizonMask.isChecked():
            # delete a point
            if len(horizon) > 2:
                del(horizon[ind])
            # now redraw plot
            self.maskPlotMarker.set_data([i[0] for i in horizon], [i[1] for i in horizon])
            x = [i[0] for i in horizon]
            x.insert(0, 0)
            x.append(360)
            y = [i[1] for i in horizon]
            y.insert(0, 0)
            y.append(0)
            self.maskPlotFill.set_xy(numpy.column_stack((x, y)))
        if event.button == 1 and ind is None and self.ui.checkEditHorizonMask.isChecked():
            if indlow is not None:
                horizon.insert(indlow + 1, (event.xdata, event.ydata))
            self.maskPlotMarker.set_data([i[0] for i in horizon], [i[1] for i in horizon])
            x = [i[0] for i in horizon]
            x.insert(0, 0)
            x.append(360)
            y = [i[1] for i in horizon]
            y.insert(0, 0)
            y.append(0)
            self.maskPlotFill.set_xy(numpy.column_stack((x, y)))

        # finally redraw
        self.drawCanvas()

    @staticmethod
    def get_ind_under_point(event, epsilon, xy):
        if len(xy) == 0:
            return None
        xt = numpy.asarray([i[0] for i in xy])
        yt = numpy.asarray([i[1] for i in xy])
        d = numpy.sqrt((xt - event.xdata)**2 / 16 + (yt - event.ydata)**2)
        ind = d.argsort()[:1][0]
        if d[ind] >= epsilon:
            ind = None
        return ind

    @staticmethod
    def get_two_ind_under_point_in_x(event, xy):
        if len(xy) < 2:
            return None
        xt = [i[0] for i in xy]
        return bisect.bisect_left(xt, event.xdata) - 1

    def drawHemisphere(self):
        for i in range(0, len(self.annotate)):
            self.annotate[i].remove()
        self.annotate = list()
        self.starsAnnotate = list()
        # star plane
        self.hemisphereMatplotlibStar.axes.cla()
        self.hemisphereMatplotlibStar.fig.canvas.mpl_connect('button_press_event', self.onMouse)
        self.hemisphereMatplotlibStar.axes.set_facecolor((0, 0, 0, 0))
        self.hemisphereMatplotlibStar.axes.set_xlim(0, 360)
        self.hemisphereMatplotlibStar.axes.set_ylim(0, 90)
        self.hemisphereMatplotlibStar.axes.set_axis_off()
        starsTopo = self.app.workerMountDispatcher.data['starsTopo']
        starsNames = self.app.workerMountDispatcher.data['starsNames']
        self.starsAlignment,  = self.hemisphereMatplotlibStar.axes.plot([i[0] for i in starsTopo], [i[1] for i in starsTopo], '*', markersize=6, color='#C0C000')
        for i in range(0, len(starsTopo)):
            self.starsAnnotate.append(self.hemisphereMatplotlibStar.axes.annotate(starsNames[i],
                                                                                  xy=(starsTopo[i][0] + self.offx,
                                                                                      starsTopo[i][1] + self.offy),
                                                                                  xycoords=('data', 'data'),
                                                                                  color='#808080', fontsize=12, clip_on=True))

        # moving widget plane
        self.hemisphereMatplotlibMoving.axes.cla()
        self.hemisphereMatplotlibMoving.fig.canvas.mpl_connect('button_press_event', self.onMouse)
        self.hemisphereMatplotlibMoving.axes.set_facecolor((0, 0, 0, 0))
        self.hemisphereMatplotlibMoving.axes.set_xlim(0, 360)
        self.hemisphereMatplotlibMoving.axes.set_ylim(0, 90)
        self.hemisphereMatplotlibMoving.axes.set_axis_off()

        # fixed points and horizon plane
        self.hemisphereMatplotlib.axes.cla()
        self.hemisphereMatplotlib.fig.canvas.mpl_connect('button_press_event', self.onMouse)
        self.hemisphereMatplotlib.axes.spines['bottom'].set_color('#2090C0')
        self.hemisphereMatplotlib.axes.spines['top'].set_color('#2090C0')
        self.hemisphereMatplotlib.axes.spines['left'].set_color('#2090C0')
        self.hemisphereMatplotlib.axes.spines['right'].set_color('#2090C0')
        self.hemisphereMatplotlib.axes.grid(True, color='#404040')
        self.hemisphereMatplotlib.axes.set_facecolor((0, 0, 0, 0))
        self.hemisphereMatplotlib.axes.tick_params(axis='x', colors='#2090C0', labelsize=12)
        self.hemisphereMatplotlib.axes.set_xlim(0, 360)
        self.hemisphereMatplotlib.axes.set_xticks(numpy.arange(0, 361, 30))
        self.hemisphereMatplotlib.axes.set_ylim(0, 90)
        self.hemisphereMatplotlib.axes.tick_params(axis='y', colors='#2090C0', which='both', labelleft='on', labelright='on', labelsize=12)
        self.hemisphereMatplotlib.axes.set_xlabel('Azimuth in degrees', color='#2090C0', fontweight='bold', fontsize=12)
        self.hemisphereMatplotlib.axes.set_ylabel('Altitude in degrees', color='#2090C0', fontweight='bold', fontsize=12)
        # horizon
        horizon = self.app.workerModelingDispatcher.modelingRunner.modelPoints.horizonPoints
        if len(horizon) < 2:
            del(horizon[:])
            horizon.append((0, 0))
            horizon.append((360, 0))
        x = [i[0] for i in horizon]
        x.insert(0, 0)
        x.append(360)
        y = [i[1] for i in horizon]
        y.insert(0, 0)
        y.append(0)
        self.maskPlotFill,  = self.hemisphereMatplotlib.axes.fill(x, y, color='#002000', zorder=-20)
        self.maskPlotMarker,  = self.hemisphereMatplotlib.axes.plot([i[0] for i in horizon], [i[1] for i in horizon], color='#006000', zorder=-20, lw=3)
        if self.ui.checkEditHorizonMask.isChecked():
            self.maskPlotMarker.set_marker('o')
            self.maskPlotMarker.set_color('#FF00FF')
        # model points
        points = self.app.workerModelingDispatcher.modelingRunner.modelPoints.modelPoints
        # draw points in two colors
        self.pointsPlotBig,  = self.hemisphereMatplotlib.axes.plot([i[0] for i in points], [i[1] for i in points], 'o', markersize=9, fillstyle='none', color='#00A000')
        self.pointsPlotSmall,  = self.hemisphereMatplotlib.axes.plot([i[0] for i in points], [i[1] for i in points], 'o', markersize=3, color='#E0E000', picker='None')
        if self.ui.checkEditModelPoints.isChecked():
            self.pointsPlotBig.set_color('#FF00FF')
        # add text to points
        for i in range(0, len(points)):
            self.annotate.append(self.hemisphereMatplotlib.axes.annotate('{0:2d}'.format(i+1), xy=(points[i][0] + self.offx, points[i][1] + self.offy), color='#E0E0E0'))
        # add crosses, if modeling was done to recap when opening the window
        self.pointsPlotCross, = self.hemisphereMatplotlib.axes.plot([], [], 'x', color='#FF0000', zorder=5, markersize=9, lw=2)
        # draw celestial equator
        celestial = self.app.workerModelingDispatcher.modelingRunner.modelPoints.celestialEquator
        self.celestial,  = self.hemisphereMatplotlib.axes.plot([i[0] for i in celestial], [i[1] for i in celestial], '.', markersize=1, fillstyle='none', color='#808080', visible=False)
        # draw meridian limits
        self.deltaGuide = matplotlib.patches.Rectangle((180, 0), 1, 90, zorder=-10, color='#FFFF0040', lw=1, fill=True, visible=False)
        self.deltaSlew = matplotlib.patches.Rectangle((180, 0), 1, 90, zorder=-10, color='#FF000040', lw=1, fill=True, visible=False)
        self.hemisphereMatplotlib.axes.add_patch(self.deltaGuide)
        self.hemisphereMatplotlib.axes.add_patch(self.deltaSlew)

        # now to the third widget on top of the other ones
        # adding the pointer of mount to hemisphereMoving plot
        self.pointerAzAlt1,  = self.hemisphereMatplotlibMoving.axes.plot(180, 45, zorder=10, color='#FF00FF', marker='o', markersize=25, markeredgewidth=3, fillstyle='none', visible=False)
        self.pointerAzAlt2,  = self.hemisphereMatplotlibMoving.axes.plot(180, 45, zorder=10, color='#FF00FF', marker='o', markersize=10, markeredgewidth=1, fillstyle='none', visible=False)
        self.pointerAzAlt3,  = self.hemisphereMatplotlibMoving.axes.plot(180, 45, zorder=10, color='#FF00FF', marker='.', markersize=2, markeredgewidth=1, fillstyle='none', visible=False)
        # adding pointer of dome if dome is present
        self.pointerDome1 = matplotlib.patches.Rectangle((165, 1), 30, 88, zorder=-30, color='#40404080', lw=3, fill=True, visible=False)
        self.pointerDome2 = matplotlib.patches.Rectangle((165, 1), 30, 88, zorder=-30, color='#80808080', lw=3, fill=False, visible=False)
        self.hemisphereMatplotlibMoving.axes.add_patch(self.pointerDome1)
        self.hemisphereMatplotlibMoving.axes.add_patch(self.pointerDome2)

        # drawing the whole stuff
        self.setOperationModus()
        self.resizeEvent(0)



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.4
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import logging
import socket


class CheckIP:
    logger = logging.getLogger(__name__)

    def __init__(self):
        pass

    def checkIPAvailable(self, Host, Port):
        returnValue = False
        try:
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            sock.settimeout(2)
            sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            result = sock.connect_ex((Host, Port))
            if result == 0:
                returnValue = True
            else:
                returnValue = False
        except socket.gaierror:
            self.logger.info('Host address could not be resolved: host {0}:{1}'.format(Host, Port))
        except Exception as e:
            self.logger.error('Error checking host {0}:{1}, error: {2}'.format(Host, Port, e))
        finally:
            sock.close()
        return returnValue



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.4
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import logging
import PyQt5
import time


class MountCommandRunner(PyQt5.QtCore.QObject):
    logger = logging.getLogger(__name__)

    CONNECTION_TIMEOUT = 2000
    CYCLE = 250
    signalDestruct = PyQt5.QtCore.pyqtSignal()
    # define the number of bytes for the return bytes in case of not having them in bulk mode
    # this is needed, because the mount computer  doesn't support a transaction base like number of
    # bytes to be expected. it's just plain data and i have to find out myself how much it is.
    # due to the fact i'm doing multi threading with multi connections some of the commands run in parallel
    COMMAND_RETURN = {':AP#': 0,
                      ':hP#': 0,
                      ':PO#': 0,
                      ':RT0#': 0,
                      ':RT1#': 0,
                      ':RT2#': 0,
                      ':RT9#': 0,
                      ':STOP#': 0,
                      ':U2#': 0,
                      ':modelld0': 2,
                      ':modelsv0': 2,
                      ':modeldel0': 2,
                      ':delalst': 2,
                      ':delalig': 1,
                      ':SRPRS': 1,
                      ':SRTMP': 1,
                      ':Sz': 1,
                      ':Sa': 1,
                      ':Sr': 1,
                      ':Sd': 1,
                      ':MA#': 1,
                      ':MS#': 1,
                      ':shutdown': 1,
                      ':Sw': 1,
                      ':Sdat': 1,
                      ':Suaf': 1,
                      ':FLIP': 1,
                      ':So': 1,
                      ':Sh': 1,
                      ':SREF': 1,
                      ':CM#': 27,
                      ':CMS#': 1,
                      ':Gr#': 12,
                      ':Gd#': 12,
                      ':newalig#': 1,
                      ':endalig#': 1,
                      ':newalpt': 1,
                      ':CMCFG': 1}

    def __init__(self, app, thread, data, signalConnected, mountStatus):
        super().__init__()

        self.app = app
        self.thread = thread
        self.data = data
        self.signalConnected = signalConnected
        self.mountStatus = mountStatus
        self.mutexIsRunning = PyQt5.QtCore.QMutex()
        self.isRunning = False
        self.connectCounter = 0
        self.socket = None
        self.sendLock = False
        self.cycleTimer = None
        self.messageString = ''
        self.numberBytesToReceive = -1
        self.commandSet = dict()

    def run(self):
        self.logger.info('mount command started')
        self.mutexIsRunning.lock()
        if not self.isRunning:
            self.isRunning = True
        self.mutexIsRunning.unlock()
        self.socket = PyQt5.QtNetwork.QTcpSocket()
        self.socket.setSocketOption(PyQt5.QtNetwork.QAbstractSocket.LowDelayOption, 1)
        self.socket.setSocketOption(PyQt5.QtNetwork.QAbstractSocket.KeepAliveOption, 1)
        self.socket.hostFound.connect(self.handleHostFound)
        self.socket.connected.connect(self.handleConnected)
        self.socket.stateChanged.connect(self.handleStateChanged)
        self.socket.disconnected.connect(self.handleDisconnect)
        self.socket.error.connect(self.handleError)
        self.socket.readyRead.connect(self.handleReadyRead)

        self.signalDestruct.connect(self.destruct, type=PyQt5.QtCore.Qt.BlockingQueuedConnection)
        self.cycleTimer = PyQt5.QtCore.QTimer(self)
        self.cycleTimer.setSingleShot(False)
        self.cycleTimer.timeout.connect(self.doCommand)
        self.cycleTimer.start(self.CYCLE)

    def stop(self):
        self.mutexIsRunning.lock()
        if self.isRunning:
            self.isRunning = False
            self.signalDestruct.emit()
            self.signalConnected.emit({'Command': False})
            self.thread.quit()
            self.thread.wait()
        self.mutexIsRunning.unlock()
        self.logger.info('mount command stopped')

    @PyQt5.QtCore.pyqtSlot()
    def destruct(self):
        if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
            self.socket.disconnectFromHost()
        self.cycleTimer.stop()
        self.signalDestruct.disconnect(self.destruct)
        self.socket.hostFound.disconnect(self.handleHostFound)
        self.socket.connected.disconnect(self.handleConnected)
        self.socket.stateChanged.disconnect(self.handleStateChanged)
        self.socket.disconnected.disconnect(self.handleDisconnect)
        self.socket.error.disconnect(self.handleError)
        self.socket.readyRead.disconnect(self.handleReadyRead)
        self.socket.abort()

    def doCommand(self):
        self.doReconnect()
        if not self.app.mountCommandQueue.empty() and (self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState) and not self.sendLock:
            rawCommand = self.app.mountCommandQueue.get()
            if isinstance(rawCommand, str):
                # only a single command without return needed
                command = rawCommand
            elif isinstance(rawCommand, dict):
                self.commandSet = rawCommand
                command = rawCommand['command']
            else:
                command = ''
                self.logger.error('Mount RunnerCommand received command {0} wrong type: {1}'.format(rawCommand, type(rawCommand)))
            if len(command) > 0:
                # determine how many bytes to receive
                self.numberBytesToReceive = -1
                for key in self.COMMAND_RETURN:
                    if command.startswith(key):
                        self.numberBytesToReceive = self.COMMAND_RETURN[key]
                        break
                if self.numberBytesToReceive == -1:
                    self.logger.error('Command >(0)< not known'.format(command))
                elif self.numberBytesToReceive > 0:
                    self.sendLock = True
                    self.sendCommand(command)
                else:
                    self.sendLock = False
                    self.sendCommand(command)

    def doReconnect(self):
        # to get order in connections, we wait for first connecting the once type
        if self.mountStatus['Once'] and self.data['FW'] > 0:
            if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.UnconnectedState:
                if self.connectCounter == 0:
                    self.app.sharedMountDataLock.lockForRead()
                    self.socket.connectToHost(self.data['MountIP'], self.data['MountPort'])
                    self.app.sharedMountDataLock.unlock()
                else:
                    # connection build up is ongoing
                    pass
                if self.connectCounter * self.CYCLE > self.CONNECTION_TIMEOUT:
                    self.socket.abort()
                    self.connectCounter = 0
                else:
                    self.connectCounter += 1
            else:
                if self.socket.state() != PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
                    if self.connectCounter * self.CYCLE > self.CONNECTION_TIMEOUT:
                        self.socket.abort()
                        self.connectCounter = 0
                    else:
                        self.connectCounter += 1
                else:
                    # connected
                    pass

    @PyQt5.QtCore.pyqtSlot()
    def handleHostFound(self):
        self.app.sharedMountDataLock.lockForRead()
        self.logger.debug('Mount RunnerCommand found at {}:{}'.format(self.data['MountIP'], self.data['MountPort']))
        self.app.sharedMountDataLock.unlock()

    @PyQt5.QtCore.pyqtSlot()
    def handleConnected(self):
        self.signalConnected.emit({'Command': True})
        self.app.sharedMountDataLock.lockForRead()
        self.logger.info('Mount RunnerCommand connected at {0}:{1}'.format(self.data['MountIP'], self.data['MountPort']))
        self.app.sharedMountDataLock.unlock()

    @PyQt5.QtCore.pyqtSlot(PyQt5.QtNetwork.QAbstractSocket.SocketError)
    def handleError(self, socketError):
        self.logger.warning('Mount RunnerCommand connection fault: {0}'.format(socketError))

    @PyQt5.QtCore.pyqtSlot()
    def handleStateChanged(self):
        self.logger.debug('Mount RunnerCommand connection has state: {0}'.format(self.socket.state()))

    @PyQt5.QtCore.pyqtSlot()
    def handleDisconnect(self):
        self.logger.info('Mount RunnerCommand connection is disconnected from host')
        self.signalConnected.emit({'Command': False})

    @PyQt5.QtCore.pyqtSlot()
    def handleReadyRead(self):
        while (len(self.messageString) < self.numberBytesToReceive) and self.isRunning:
            self.messageString += self.socket.read(1024).decode()
        messageToProcess = self.messageString
        self.messageString = ''
        self.commandSet['reply'] = messageToProcess.rstrip('#')
        self.sendLock = False

    def sendCommand(self, command):
        if self.isRunning:
            if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
                self.socket.write(bytes(command + '\r', encoding='ascii'))
                self.socket.flush()
            else:
                self.sendLock = False
                self.logger.warning('Socket CommandRunner not connected')



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.5
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import json
import logging
import platform
import time
import PyQt5
import requests
from baseclasses import checkIP


class SGPro:
    logger = logging.getLogger(__name__)

    # base definitions of class
    host = '127.0.0.1'
    port = 59590
    ipSGProBase = 'http://' + host + ':' + str(port)
    ipSGPro = 'http://' + host + ':' + str(port) + '/json/reply/'

    getDeviceStatusPath = 'SgGetDeviceStatus'
    getSolvedImageDataPath = 'SgGetSolvedImageData'
    solveImagePath = 'SgSolveImage'

    ASTROMETRY_STATUS = {'ERROR': 'ERROR', 'DISCONNECTED': 'DISCONNECTED', 'IDLE': 'IDLE', 'BUSY': 'BUSY'}

    def __init__(self, main, app, data):
        # make main sources available
        self.main = main
        self.app = app
        self.data = data
        self.cancel = False
        self.mutexCancel = PyQt5.QtCore.QMutex()
        self.checkIP = checkIP.CheckIP()

        self.application = dict()
        self.application['Available'] = False
        self.application['Name'] = ''
        self.application['InstallPath'] = ''
        self.application['Status'] = ''
        self.application['Runtime'] = 'Sequence Generator.exe'

        if platform.system() == 'Windows':
            # sgpro only supported on local machine
            self.application['Available'], self.application['Name'], self.application['InstallPath'] = self.app.checkRegistrationKeys('Sequence Generator')
            if self.application['Available']:
                self.app.messageQueue.put('Found Astrometry: {0}\n'.format(self.application['Name']))
                self.logger.info('Name: {0}, Path: {1}'.format(self.application['Name'], self.application['InstallPath']))
            else:
                self.logger.info('Application SGPro not found on computer')

    def initConfig(self):
        try:
            if 'UseBlindSolving' in self.app.config:
                self.app.ui.checkUseBlindSolving.setChecked(self.app.config['UseBlindSolving'])
        except Exception as e:
            self.logger.error('Item in config.cfg for sgpro astrometry client could not be initialized, error:{0}'.format(e))
        finally:
            pass

    def storeConfig(self):
        self.app.config['UseBlindSolving'] = self.app.ui.checkUseBlindSolving.isChecked()

    def start(self):
        pass

    def stop(self):
        pass

    def getStatus(self):
        if self.checkIP.checkIPAvailable(self.host, self.port):
            suc, state, message = self.SgGetDeviceStatus('PlateSolver')
            if suc:
                self.application['Status'] = 'OK'
                if state in self.ASTROMETRY_STATUS:
                    self.data['Status'] = self.ASTROMETRY_STATUS[state]
                    if self.ASTROMETRY_STATUS[state] == 'DISCONNECTED':
                        self.data['CONNECTION']['CONNECT'] = 'Off'
                    else:
                        self.data['CONNECTION']['CONNECT'] = 'On'
                else:
                    self.logger.error('Unknown solver status: {0}'.format(state))
            else:
                self.application['Status'] = 'ERROR'
                self.data['CONNECTION']['CONNECT'] = 'Off'
        else:
            self.application['Status'] = 'ERROR'
            self.data['CONNECTION']['CONNECT'] = 'Off'

    def solveImage(self, imageParams):
        self.mutexCancel.lock()
        self.cancel = False
        self.mutexCancel.unlock()

        # waiting for start solving
        timeSolvingStart = time.time()
        self.main.astrometryStatusText.emit('START')
        self.main.astrometrySolvingTime.emit('{0:02.0f}'.format(time.time() - timeSolvingStart))
        if self.app.ui.checkUseBlindSolving.isChecked():
            suc, mes, guid = self.SgSolveImage(imageParams['Imagepath'],
                                               BlindSolve=True,
                                               UseFitsHeaders=True)
        else:
            suc, mes, guid = self.SgSolveImage(imageParams['Imagepath'],
                                               BlindSolve=False,
                                               UseFitsHeaders=True)
        if not suc:
            self.logger.warning('Solver no start, message: {0}'.format(mes))
            self.main.astrometryStatusText.emit('ERROR')
            imageParams['Imagepath'] = ''
            imageParams['Message'] = mes
            return

        # loop for upload
        self.main.astrometryStatusText.emit('UPLOAD')
        while not self.cancel:
            suc, state, mes = self.SgGetDeviceStatus('PlateSolver')
            if 'IDLE' in state:
                break
            self.main.astrometrySolvingTime.emit('{0:02.0f}'.format(time.time()-timeSolvingStart))
            time.sleep(0.1)

        # loop for solve
        self.main.astrometryStatusText.emit('SOLVE')
        while not self.cancel:
            solved, mes, ra_sol, dec_sol, scale, angle, timeTS = self.SgGetSolvedImageData(guid)
            if solved:
                # solved gives the status for PinPoint
                break
            if 'ailed' in mes:
                # Failed or failed is in PlanWave, Astrometry
                imageParams['Message'] = mes
                break
            self.main.astrometrySolvingTime.emit('{0:02.0f}'.format(time.time()-timeSolvingStart))
            time.sleep(0.1)
        imageParams['Solved'] = solved

        # Loop for data
        self.main.imageSolved.emit()
        self.main.astrometryStatusText.emit('GET DATA')
        while not self.cancel and solved:
            if solved:
                imageParams['RaJ2000Solved'] = float(ra_sol)
                imageParams['DecJ2000Solved'] = float(dec_sol)
                imageParams['Scale'] = float(scale)
                imageParams['Angle'] = float(angle)
                imageParams['TimeTS'] = float(timeTS)
                imageParams['Message'] = 'Solved'
                break
            else:
                imageParams['RaJ2000Solved'] = 0
                imageParams['DecJ2000Solved'] = 0
                imageParams['Scale'] = 0
                imageParams['Angle'] = 0
                imageParams['TimeTS'] = 0
                break
            time.sleep(0.1)
            self.main.astrometrySolvingTime.emit('{0:02.0f}'.format(time.time()-timeSolvingStart))

        # finally idle
        self.main.imageDataDownloaded.emit()
        self.main.astrometryStatusText.emit('IDLE')
        self.main.astrometrySolvingTime.emit('')

    def SgGetDeviceStatus(self, device):
        # reference {"Device": "Camera"}, devices are "Camera", "FilterWheel", "Focuser", "Telescope" and "PlateSolver"}
        data = {'Device': device}
        try:
            result = requests.post(self.ipSGPro + self.getDeviceStatusPath, data=bytes(json.dumps(data).encode('utf-8')))
            result = json.loads(result.text)
            if 'Message' not in result:
                result['Message'] = 'None'
            return result['Success'], result['State'], result['Message']
        except Exception as e:
            self.logger.error('error: {0}'.format(e))
            return False, 'Request failed', 'Request failed'

    def SgGetSolvedImageData(self, _guid):
        # reference {"Receipt":"00000000000000000000000000000000"}
        data = {'Receipt': _guid}
        try:
            result = requests.post(self.ipSGPro + self.getSolvedImageDataPath, data=bytes(json.dumps(data).encode('utf-8')))
            result = json.loads(result.text)
            return result['Success'], result['Message'], result['Ra'], result['Dec'], result['Scale'], result['Angle'], result['TimeToSolve']
        except Exception as e:
            self.logger.error('error: {0}'.format(e))
            return False, 'Request failed', '', '', '', '', ''

    def SgSolveImage(self, path, RaHint=None, DecHint=None, ScaleHint=None, BlindSolve=False, UseFitsHeaders=False):
        # reference {"ImagePath":"String","RaHint":0,"DecHint":0,"ScaleHint":0,"BlindSolve":false,"UseFitsHeadersForHints":false}
        data = {"ImagePath": path, "BlindSolve": BlindSolve, "UseFitsHeadersForHints": UseFitsHeaders}
        if RaHint:
            data['RaHint'] = RaHint
        if DecHint:
            data['DecHint'] = DecHint
        if ScaleHint:
            data['ScaleHint'] = ScaleHint
        try:
            result = requests.post(self.ipSGPro + self.solveImagePath, data=bytes(json.dumps(data).encode('utf-8')))
            result = json.loads(result.text)
            return result['Success'], result['Message'], result['Receipt']
        except Exception as e:
            self.logger.error('error: {0}'.format(e))
            return False, 'Request failed', ''
